package org.ksv.plugin;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * Responsible for copy history.
 * Keeps all copy operations.
 * Serialize & deserialize copy history 
 * @author ksv
 *
 */
public class CopyHistory {
	/**
	 * Saves file's copy history
	 * @param fileTo 	
	 * @param fileFrom
	 */
	public static void setCopyHistory( File fileTo, File fileFrom ) {
		
		String fileFromStr = fileFrom.getAbsolutePath();
		
		// Check whether the file is being copied for the second time
		// If it is, save its original value
		String origFileStr = copyHistory.get( fileFromStr );
		
		if ( origFileStr == null ) {
			origFileStr = fileFromStr;
		}
		else {
			copyHistory.remove( fileFromStr );
		}
		
		copyHistory.put( fileTo.getAbsolutePath(), origFileStr );		
	}
	
	/**
	 * Gets file's copy history
	 * @param file
	 * @return
	 */
	public static File getFromCopyHistory ( File file ) {
		String fileFromStr = copyHistory.get( file.getAbsolutePath() );
		if ( fileFromStr == null ) {
			return null;
		}
		return new File( fileFromStr );
	}
	
	/**
	 * Serializes copy history 
	 * @throws IOException 
	 */
	public static void serializeCopyHistory() throws IOException {		
		
		// Remove junk entries related to not opened files		
		// Check if an entry is in the project. If it's not, remove the entry
		for (Iterator<String> i = copyHistory.keySet().iterator(); i.hasNext();) {			
			String filename = i.next();
 			if ( KsvUtils.getLastProject().file2resource( filename ) == null ) {
				i.remove();
			}
		}
		
		// Serialize the actual history
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		try
		{
			fos = new FileOutputStream( COPY_HISTORY_FILENAME );
			out = new ObjectOutputStream( fos );
			out.writeObject( copyHistory );			
		}
		catch( Exception ex ) {			
		}
		finally {
			if ( out != null ) out.close();
			if ( fos != null ) fos.close();
		}
	}
	
	@SuppressWarnings("unchecked")
	public static void deserializeCopyHistory() throws IOException {
		FileInputStream fis = null;
		ObjectInputStream in = null;
		try
		{
			fis = new FileInputStream( COPY_HISTORY_FILENAME );
			in = new ObjectInputStream( fis );
			copyHistory = (Hashtable<String, String>) in.readObject();		
		}
		catch(Exception ex) {
		}
		finally {
			if ( in != null ) in.close();
			if ( fis != null ) fis.close();
		}		
	}
	
	static Hashtable<String,String> copyHistory = new Hashtable<String,String>();
	static final String COPY_HISTORY_FILENAME = Activator.getDefault().getStateLocation().addTrailingSeparator().append( "copyhistory.ser" ).toString();

}
