package org.ksv.plugin;

import java.io.File;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.actions.CopyFilesAndFoldersOperation;
import com.openedge.pdt.text.OEProject;

/**
 * Copies an external file into the current OpenEdge project
 * @author ksv
 *
 */
public class CopyToProject implements IWorkbenchWindowActionDelegate, IKsvPlugin{

	private IWorkbenchWindow window;
	
	public void dispose() {
		// TODO Auto-generated method stub		
	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
		
	}

	public void run(IAction action) {
		KsvUtils.runPlugin(this);		
	}
	
	public void selectionChanged(IAction action, ISelection selection) {		
	}

	/**
	 * Copies the current file into the current OE Project 
	 */
	public void runAction(OEProject project) {
		
		// Get the current file opened in the editor
		File file = KsvUtils.getEditorFile(window);
		
		// If file is ok, try to copy it
		if (file != null && file.exists()) {
			
			// Check if the file is already in the project
			IProject prj = KsvUtils.getLastProject().getProject();			
			if (KsvUtils.getActiveProject() != prj) {
				// Create a copy operation
				CopyFilesAndFoldersOperation cp = new CopyFilesAndFoldersOperation(window.getShell());
				// Copy
				cp.copyFiles(new String[] { file.getAbsolutePath() }, prj);
				// Look for the file in the project and reopen it in the editor
				try {
					File copiedFile = new File(prj.getFile(file.getName()).getLocationURI());
					if (KsvUtils.closeFileInCurrentEditor()) {
						KsvUtils.openFileInEditor( copiedFile );
					}				
					// Add the original file into history
	    			CopyHistory.setCopyHistory( copiedFile, file );
					prj.refreshLocal(0, null);
				}
				catch (CoreException e)
				{
					MessageDialog.openError(
							window.getShell(), 
							"KsvPlugin", 
							"File [" + file.getName() + "] cannot be copied.\n\n" + e.getMessage());
				}
			}
			else {
				MessageDialog.openInformation (
						window.getShell(), 
						"KsvPlugin", 
						"File [" + file.getName() + "] is already in the project");
			}

		}	
	}

}
