package org.ksv.plugin;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.*;
import com.openedge.pdt.text.OEProject;

/**
 * Looks for a specified file using OpenEdge PROPATH and
 * opens it in the editor  
 * @author ksv
 *
 */
public class OpenInPropath implements IWorkbenchWindowActionDelegate, IKsvPlugin {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public OpenInPropath() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {		
		KsvUtils.runPlugin(this);
	}

	/**
	 * Opens file using current OE project PROPATH
	 * @param project - current OE project
	 */
	public void runAction(OEProject project) {

		boolean fileOpened = false;
		
		String filename = "";
		
		// Loop while either a file will be opened or user will cancel it  
		do {
			// Ask a user to type a filename using a common fileopen dialog
			filename = KsvUtils.openFileDialog(window,"Open file using PROPATH",new String[] { "*.*" },filename);
			fileOpened = filename != null ? KsvUtils.openFileInPropath(filename, project, true, window) : true;
			
		} while (!fileOpened);

	}	

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {			
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
	
}