package org.ksv.plugin;

import java.io.File;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.openedge.pdt.text.OEProject;

/**
 * Builds project's PROPATH using a folder (root folder) and all its
 * subfolders
 * @author ksv
 *
 */
public class PropathBuilder implements IWorkbenchWindowActionDelegate, IKsvPlugin{

	private IWorkbenchWindow window;
	
	public void dispose() {
		// TODO Auto-generated method stub		
	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
		
	}

	public void run(IAction action) {
		KsvUtils.runPlugin(this);		
	}
	
	public void selectionChanged(IAction action, ISelection selection) {		
	}
	
	/**
	 * Builds project PROPATH using a root folder chosen by a user
	 */
	public void runAction(OEProject project) {
		
		// ask a user to choose a root folder
		String folder = KsvUtils.openFolderDialog( window, "Choose a root folder for PROPATH" );
		if (folder != null)
		{			
			String pathList = getListOfSubdirs( new File( folder ) );
			
			String path = ( pathList.length() > PATH_LENGTH ? pathList.substring( 0, PATH_LENGTH ) + " ..." : pathList );
			
			// Ask the user before changing PROPATH
			if ( MessageDialog.openConfirm(
					window.getShell(), 
					"KsvPlugin", 
					"The following folders are about to be added into your PROPATH:\n\n" + 
					path + 
					"\n\nDo you agree?" ) ) {
				// Change PROPATH and save it for the next runs
				KsvUtils.AddDirsToPropath( project, pathList, true );
			}
		}
	}
	
	/**
	 * Returns a folder and its subfolders as a comma-delimited list 
	 */
	private static String getListOfSubdirs ( File rootFolder ) {
		
		File[] files = rootFolder.listFiles();		
		String result = rootFolder.getAbsolutePath();
		
		for( File file : files ) {
			if ( file.isDirectory() ) {				
				result += "," + getListOfSubdirs( file ); 								
			}
		}
		return result;
	}
	
	/**
	 * An amount of PROPATH to show to a user 
	 */
	final static int PATH_LENGTH = 1024; 

}
